from enum import Enum


class SoftwareName(Enum):
    ALERTSITE_MONITORING_BOT = 'alertsite-monitoring-bot'
    ANDROID = 'android-browser'
    AWESOMIUM = 'awesomium'
    BLACKBERRY = 'blackberry-browser'
    CATCHPOINT_ANALYSER = 'catchpoint-analyser'
    CHROME = 'chrome'
    CHROMIUM = 'chromium'
    COSMOS_CRAWLER = 'cosmos-crawler'
    DOTCOM_MONITOR_BOT = 'dotcom-monitor-bot'
    EDGE = 'edge'
    FIREFOX = 'firefox'
    FIREFOX_FOCUS = 'firefox-focus'
    GOOGLE_APP_ENGINE = 'google-app-engine-software'
    INTERNET_ARCHIVER_BOT = 'internet-archiver-bot'
    INTERNET_CHANNEL = 'internet-channel'
    INTERNET_TV = 'internet-tv-browser'
    K_MELEON = 'k-meleon'
    NETSCAPE_NAVIGATOR = 'netscape-navigator'
    NETSURF = 'netsurf'
    NOKIA = 'nokia-browser'
    OBIGO = 'obigo'
    OMNIWEB = 'omniweb'
    ONEBROWSER = 'onebrowser'
    OPERA = 'opera'
    OPERA_MINI = 'opera-mini'
    PALE_MOON = 'pale-moon'
    QUPZILLA = 'qupzilla'
    ROCKMELT = 'rockmelt'
    SKYFIRE = 'skyfire'
    SPEEDCURVE_SPEED_TESTER = 'speedcurve-speed-tester'
    UC_BROWSER = 'uc-browser'
    WEBKIT = 'webkit-based-browser'
    WEBTV = 'webtv'
    YANDEX = 'yandex-browser'
    YANDEX_SEARCH_BOT = 'yandex-search-bot'
    YODAOBOT_SEARCH_BOT = 'yodaobot-search-bot'
    BLAZER = 'blazer'
    AMAZON_API_GATEWAY = 'amazon-api-gateway'
    ALEXA_BOT = 'alexa-bot'
    GOOGLE_EARTH = 'google-earth'
    EVALIANT_IMPRESSIONS_BOT = 'evaliant-impressions-bot'
    PLAYBOOK_WEB_BROWSER = 'playbook-web-browser'
    FACEBOOK_APP = 'facebook-app'
    ELECTRON_APPLICATION = 'electron-application'
    ONE_TWENTY_SIX_BROWSER = '126-browser'
    DUCKDUCKGO_FAVICONS_BOT = 'duckduckgo-favicons-bot'
    PYTHON_URLLIB = 'python-urllib'
    NETFRONT_BROWSER_NX = 'netfront-browser-nx'
    AVANT_BROWSER = 'avant-browser'
    MERCURY_BROWSER = 'mercury-browser'
    FLOCK = 'flock'
    VALVE_STEAM_TENFOOT_DISPLAY = 'valve-steam-tenfoot-display'
    CHARLOTTE = 'charlotte'
    KINDLE_BROWSER = 'kindle-browser'
    TEASHARK = 'teashark'
    GOMEZAGENT = 'gomezagent'
    QQ_BROWSER = 'qq-browser'
    OFFICE = 'office'
    BROWSERSHOTS_COM_CROSS_BROWSER_TESTER = 'browsershots-com-cross-browser-tester'
    THUNDERBIRD = 'thunderbird'
    WYZO = 'wyzo'
    GOOGLE_FAVICON_CRAWLER = 'google-favicon-crawler'
    SECONDLIFE = 'secondlife'
    INTERNET_EXPLORER_MOBILE = 'internet-explorer-mobile'
    SONGBIRD = 'songbird'
    EIGHTY_LEGS_WEB_CRAWLER = '80legs-web-crawler'
    SQUIDER_BOT = 'squider-bot'
    SONY_WEB_BROWSER = 'sony-web-browser'
    MAXTHON = 'maxthon'
    FAST_ENTERPRISE_CRAWLER = 'fast-enterprise-crawler'
    WORD = 'word'
    ACCOONA_BUSINESS_CRAWLER = 'accoona-business-crawler'
    WEBVIEW_BASED_BROWSER = 'webview-based-browser'
    PLAYSTATION_4_BROWSER = 'playstation-4-browser'
    INTERNET_EXPLORER = 'internet-explorer'
    ELINKS = 'elinks'
    SLIMBROWSER = 'slimbrowser'
    ONENOTE = 'onenote'
    DIFFBOT_SCANNER = 'diffbot-scanner'
    DELPHI_EMBEDDED_WEB_BROWSER = 'delphi-embedded-web-browser'
    SPRAYCAN = 'spraycan'
    GOOGLE_EARTH_PRO = 'google-earth-pro'
    TENFOURFOX = 'tenfourfox'
    DOLFIN = 'dolfin'
    NOOK_WEB_BROWSER = 'nook-web-browser'
    ALEXA_SITE_AUDIT = 'alexa-site-audit'
    MOTOROLA_INTERNET_BROWSER = 'motorola-internet-browser'
    W3C_LINK_CHECKER = 'w3c-link-checker'
    PINTEREST_BOT = 'pinterest-bot'
    SOGOU_SEARCH_DOG = 'sogou-search-dog'
    FRONTPAGE = 'frontpage'
    BROADSIGN_XPRESS = 'broadsign-xpress'
    YAHOO_SLURP_WEB_CRAWLER_BOT = 'yahoo-slurp-web-crawler-bot'
    COMODO_DRAGON = 'comodo-dragon'
    EPIPHANY = 'epiphany'
    DOTNETDOTCOMDOTORG_CRAWLER = 'dotnetdotcomdotorg-crawler'
    ICAB = 'icab'
    VIENNA = 'vienna'
    RUXITSYNTHETIC = 'ruxitsynthetic'
    OKHTTP = 'okhttp'
    GNIP_UNWINDFETCHOR_CRAWLER = 'gnip-unwindfetchor-crawler'
    SILK = 'silk'
    FACEBOOK_BOT = 'facebook-bot'
    WECHAT = 'wechat'
    MICROSOFT_CRYPTOAPI = 'microsoft-cryptoapi'
    KAZEHAKASE = 'kazehakase'
    BLUE_CHROME = 'blue-chrome'
    NETCRAFT_WEB_SERVER_SURVEY = 'netcraft-web-server-survey'
    SHIRETOKO = 'shiretoko'
    GRAPESHOT_BOT = 'grapeshot-bot'
    EXCEL = 'excel'
    QTCARBROWSER = 'qtcarbrowser'
    THREE_SIXTY_SPIDER = '360spider'
    CATCHBOT = 'catchbot'
    WEBPOSITIVE = 'webpositive'
    LIBRARY_FOR_WWW_IN_PERL = 'library-for-www-in-perl'
    COMETBIRD = 'cometbird'
    SAFARI = 'safari'
    SOGOU_EXPLORER = 'sogou-explorer'
    CODA = 'coda'
    IBROWSER = 'ibrowser'
    JAVA_RUNTIME_ENVIRONMENT = 'java-runtime-environment'
    GOOGLE_IMAGE_PROXY = 'google-image-proxy'
    NEXPLAYER = 'nexplayer'
    ALIENBLUE = 'alienblue'
    MIDORI = 'midori'
    W3C_VALIDATOR = 'w3c-validator'
    PUFFIN = 'puffin'
    MAJESTIC_12_DISTRIBUTED_SEARCH_BOT = 'majestic-12-distributed-search-bot'
    ICEWEASEL = 'iceweasel'
    JAVAFX_PLATFORM = 'javafx-platform'
    CONTENT_CRAWLER_SPIDER = 'content-crawler-spider'
    COVARIO_SPIDER = 'covario-spider'
    EVE_IN_GAME_BROWSER = 'eve-in-game-browser'
    OVI = 'ovi'
    BLITZBOT_CRAWLER = 'blitzbot-crawler'
    GOOGLE_SEARCH_APP = 'google-search-app'
    BAIDU_SPIDER = 'baidu-spider'
    RAPTR = 'raptr'
    FIREBIRD = 'firebird'
    ACCOONA_AI_CRAWLER = 'accoona-ai-crawler'
    WEBOS_BROWSER = 'webos-browser'
    LINKS = 'links'
    SAMSUNG_BROWSER = 'samsung-browser'
    GALEON = 'galeon'
    CLAMAV_WEBSITE_SCANNER = 'clamav-website-scanner'
    EXB_LANGUAGE_CRAWLER = 'exb-language-crawler'
    OUTLOOK = 'outlook'
    BONECHO = 'bonecho'
    NINTENDO_BROWSER = 'nintendo-browser'
    NEWSBOT = 'newsbot'
    VIVALDI = 'vivaldi'
    VISION_MOBILE_BROWSER = 'vision-mobile-browser'
    EARTHCOM_CRAWLER = 'earthcom-crawler'
    ADOBE_AIR = 'adobe-air'
    BINGPREVIEW = 'bingpreview'
    WEBLINK_PREVIEW = 'weblink-preview'
    LYNX = 'lynx'
    COMODO_ICEDRAGON = 'comodo-icedragon'
    ICEAPE = 'iceape'
    FINDLINKS_CRAWLER_BOT = 'findlinks-crawler-bot'
    MINEFIELD = 'minefield'
    NAENARA = 'naenara'
    MSN_BOT = 'msn-bot'
    CODER_NUT = 'coder-nut'
    W3M = 'w3m'
    BIT_LY_LINK_CHECKER = 'bit-ly-link-checker'
    OPENSITEEXPLORER_CRAWLER = 'opensiteexplorer-crawler'
    BAIDU_IMAGE_SPIDER = 'baidu-image-spider'
    TWITTERBOT = 'twitterbot'
    EARTHWORM = 'earthworm'
    QT_BASED_BROWSER = 'qt-based-browser'
    THEWORLD_BROWSER = 'theworld-browser'
    FLASHFIRE = 'flashfire'
    BRAVE = 'brave'
    DISCOVERY_ENGINE_CRAWLER = 'discovery-engine-crawler'
    PECL_HTTP_LIBRARY = 'pecl-http-library'
    BESLIST_SHOPPING_CRAWLER = 'beslist-shopping-crawler'
    AVANTGO_BROWSER = 'avantgo-browser'
    GOOGLE_SNIPPET_FETCHER = 'google-snippet-fetcher'
    QUALYS_SSL_ASSESSMENT_SCANNER = 'qualys-ssl-assessment-scanner'
    OPENWAVE_MOBILE_BROWSER = 'openwave-mobile-browser'
    DOOBLE = 'dooble'
    COOLNOVO = 'coolnovo'
    ADMANTX_PLATFORM_SEMANTIC_ANALYZER = 'admantx-platform-semantic-analyzer'
    CAMINO = 'camino'
    FENNEC = 'fennec'
    SAFEPAY = 'safepay'
    BROWSER_1337 = '1337browser'
    SPLASH = 'splash'
    ARORA = 'arora'
    DRAGON = 'dragon'
    NETCAST = 'netcast'
    TENCENTTRAVELER = 'tencenttraveler'
    ITUNES = 'itunes'
    EMBEDDED_WEB_BROWSER = 'embedded-web-browser'
    BOITHO_DISTRIBUTED_CRAWLER = 'boitho-distributed-crawler'
    DOMAINTOOLS_SURVEYBOT = 'domaintools-surveybot'
    BAIDU_UNION_SPIDER = 'baidu-union-spider'
    WEBLINK = 'weblink'
    PINTEREST_APP = 'pinterest-app'
    CHROMEPLUS = 'chromeplus'
    VOILABOT_BETA = 'voilabot-beta'
    ORCA = 'orca'
    LUNASCAPE = 'lunascape'
    A_PASSION_FOR_JAZZ_MEDIA_CRAWLER = 'a-passion-for-jazz-media-crawler'
    NETFRONT = 'netfront'
    GOOGLE_SITE_VERIFIER_BOT = 'google-site-verifier-bot'
    ONE_PASSWORD = '1password'
    TOPSY_BUTTERFLY_ROBOT = 'topsy-butterfly-robot'
    CAREERBOT_SEARCH_CRAWLER = 'careerbot-search-crawler'
    OUTFORM_DIGITAL_DISPLAY = 'outform-digital-display'
    MSN_MEDIA_BOT = 'msn-media-bot'
    SRWARE_IRON = 'srware-iron'
    FAST_FRESH_CRAWLER = 'fast-fresh-crawler'
    JAKARTA_COMMONS_HTTPCLIENT = 'jakarta-commons-httpclient'
    VALVE_STEAM_GAME_OVERLAY = 'valve-steam-game-overlay'
    DATAPARKSEARCH_ENGINE = 'dataparksearch-engine'
    PROXIMIC_SEARCH = 'proximic-search'
    WATERFOX = 'waterfox'
    GOOGLE_STRUCTURED_DATA_TESTING_TOOL = 'google-structured-data-testing-tool'
    TWEETMEME_BOT = 'tweetmeme-bot'
    CURL = 'curl'
    VOILABOT = 'voilabot'
    AMAYA = 'amaya'
    ROCCAT = 'roccat'
    WGET = 'wget'
    GOOGLEBOT = 'googlebot'
    EA_ORIGIN_BROWSER = 'ea-origin-browser'
    SLACKBOT_LINK_CHECKER = 'slackbot-link-checker'
    REKONQ = 'rekonq'
    SEAMONKEY = 'seamonkey'
    TWITTER_APP = 'twitter-app'
    EXALEAD_CRAWLER = 'exalead-crawler'
    GOOGLE_WEBLIGHT_PROXY = 'google-weblight-proxy'
    KONQUEROR = 'konqueror'
    DOMAIN_RE_ANIMATOR_BOT = 'domain-re-animator-bot'
    LINKCHECK_ANALYSER = 'linkcheck-analyser'
    BAIDU_BOX_APP = 'baidu-box-app'
    LIGHT = 'light'
    PRO_ENGINEER_WILDFIRE = 'pro-engineer-wildfire'
    OPEN_WEBKIT_SHARP_BASED_BROWSER = 'open-webkit-sharp-based-browser'
    QIHOO_360 = 'qihoo-360'
    GENIEO_BOT = 'genieo-bot'
    SLEIPNIR = 'sleipnir'
    CHIMERA = 'chimera'
    MOSAIC = 'mosaic'
    POWERPOINT = 'powerpoint'
    PANSCIENT_CRAWLER = 'panscient-crawler'
    W3C_CSS_VALIDATOR = 'w3c-css-validator'
    UNKNOWN_BROWSER = 'unknown-browser'
    AOL_BROWSER = 'aol-browser'
    SOSOSPIDER_SEARCH_BOT = 'sosospider-search-bot'
    GOOGLEBOT_MOBILE = 'googlebot-mobile'
    IRIS = 'iris'
    BINGBOT = 'bingbot'
    GTMETRIX_ANALYSER = 'gtmetrix-analyser'
    DISCORD_BOT = 'discord-bot'
    DILLO = 'dillo'
    GOOGLE_S_MEDIA_PARTNERS_SYSTEM_ADSENSE = 'google-s-media-partners-system-adsense'
    HTTPCLIENT = 'httpclient'
    DLINK_BACKDOOR = 'dlink-backdoor'
    TELEGRAMBOT = 'telegrambot'
    QQDOWNLOAD_DOWNLOAD_MANAGER = 'qqdownload-download-manager'
    LB_BROWSER = 'lb-browser'
    AHREFS_BACKLINK_RESEARCH_BOT = 'ahrefs-backlink-research-bot'
    PHOENIX = 'phoenix'
    BECOME_COM_CRAWLER = 'become-com-crawler'


class OperatingSystem(Enum):
    UNIX = 'a-unix-based-os'
    ANDROID = 'android'
    BADA = 'bada'
    BEOS = 'beos'
    BLACKBERRY = 'blackberry-os'
    CHROMEOS = 'chromeos'
    FIRE_OS = 'fire-os'
    FREEBSD = 'freebsd'
    HAIKU = 'haiku'
    IOS = 'ios'
    LINUX = 'linux'
    MAC = 'mac'
    MAC_OS_X = 'mac-os-x'
    MACOS = 'macos'
    OPENBSD = 'openbsd'
    SUNOS = 'sunos'
    SYMBIAN = 'symbian'
    WEBOS = 'webos'
    WINDOWS = 'windows'
    WINDOWS_MOBILE = 'windows-mobile'
    WINDOWS_PHONE = 'windows-phone'
    HP_WEBOS = 'hp-webos'
    DARWIN = 'darwin'
    IRIX = 'irix'
    RIM_TABLET_OS = 'rim-tablet-os'
    LIVEAREA = 'livearea'
    PALMOS = 'palmos'


class SoftwareEngine(Enum):
    KHTML = 'khtml'
    GOANNA = 'goanna'
    EDGEHTML = 'edgehtml'
    TRIDENT = 'trident'
    PRESTO = 'presto'
    BLINK = 'blink'
    GECKO = 'gecko'
    WEBKIT = 'webkit'
    NETFRONT = 'netfront'


class HardwareType(Enum):
    COMPUTER = 'computer'
    LARGE_SCREEN__GAME_CONSOLE = 'large-screen -> game-console'
    LARGE_SCREEN__MEDIA_PLAYER = 'large-screen -> media-player'
    LARGE_SCREEN__TV = 'large-screen -> tv'
    MOBILE = 'mobile'
    MOBILE__EBOOK_READER = 'mobile -> ebook-reader'
    MOBILE__HANDHELD_GAME = 'mobile -> handheld-game'
    MOBILE__MUSIC_PLAYER = 'mobile -> music-player'
    MOBILE__PHONE = 'mobile -> phone'
    MOBILE__TABLET = 'mobile -> tablet'
    SERVER = 'server'
    PDA = 'pda'
    VEHICLE__CAR = 'vehicle -> car'
    LARGE_SCREEN = 'large-screen'
    MOBILE__WEARABLE__GLASSES = 'mobile -> wearable -> glasses'
    LARGE_SCREEN__BILLBOARD = 'large-screen -> billboard'


class SoftwareType(Enum):
    APPLICATION__SOFTWARE_LIBRARY = 'application -> software-library'
    BOT__ANALYSER = 'bot -> analyser'
    BOT__CRAWLER = 'bot -> crawler'
    BOT__SITE_MONITOR = 'bot -> site-monitor'
    WEB_BROWSER = 'browser -> web-browser'
    APPLICATION__MEDIA_PLAYER = 'application -> media-player'
    APPLICATION__DOWNLOAD_HELPER = 'application -> download-helper'
    APPLICATION = 'application'
    BOT__SECURITY_ANALYSER = 'bot -> security-analyser'
    BROWSER__IN_APP_BROWSER = 'browser -> in-app-browser'
    BOT__TOOL = 'bot -> tool'
    APPLICATION__PROXY = 'application -> proxy'
    APPLICATION__BILLBOARD = 'application -> billboard'


class Popularity(Enum):
    POPULAR = "very common"
    COMMON = "common"
    AVERAGE = "average"
    UNCOMMON = "uncommon"
